# Script modified by Miguel Velazquez and Ian Taylor at FlowJo 
# For use w/ SeqGeq 
options(warn = - 1)

# Check if SPADE is installed, if not then try and install
if (!require('monocle'))
{
    writeLines(" Monocle is not installed, installing...")

    if (! require('devtools'))
    {
        writeLines(" Getting devtools...")
        install.packages("devtools", repos = 'http://cran.us.r-project.org')
        library(devtools)
    }else {
        library(devtools)
    }
    writeLines(" Dowloading Monocle...")

    devtools::install_github("cole-trapnell-lab/monocle-release@master")
    if (! require('monocle'))
    {
        writeLines(" Successfully installed Monocle")
    }else {
        source("http://bioconductor.org/biocLite.R")
        biocLite()
        biocLite(c("DDRTree", "pheatmap"))
        library(devtools)
        devtools::install_github("cole-trapnell-lab/monocle-release@master")
    }
    library("monocle")
}else {
    writeLines(" Start loading monocle")
    library("monocle")
}


writeLines(" Importing CSV from SeqGeq")
# Read CSV from SeqGeq
importedCSVfile <- read.table(file = "SG_SAMPLE_DATA_PATH", sep = ",", row.names = 1, header = TRUE, check.names = FALSE, encoding = "UTF-8", stringsAsFactors = FALSE)
importedCSVfile = importedCSVfile[, colSums(importedCSVfile != 0) > 0]

writeLines(" Prepare data file...")
# Some data has numbers as cellIDs only, append a string to cellIds, remove on export.
colnames(importedCSVfile) <- paste("AppID", colnames(importedCSVfile), sep = "_")

writeLines(" Read in file")
writeLines(" Start: Writting Monocle SCE object")

writeLines(" Clean data matrix..")
expressionMatrix <- importedCSVfile
colnames(expressionMatrix) <- NULL
cellAnnotations <- data.frame(colnames(importedCSVfile))
colnames(cellAnnotations) <- c("Sample")

writeLines(" Prepare Monocle Objects..")
geneAnnotations <- data.frame(gene_short_name = rownames(importedCSVfile), row.names = rownames(importedCSVfile))

phenotypeDataframe <- new("AnnotatedDataFrame", data = cellAnnotations)
featureDataFrame <- new("AnnotatedDataFrame", data = geneAnnotations)

writeLines("Write Monocle CellDataSet object..")
cds <- newCellDataSet(as(as.matrix(expressionMatrix), 'sparseMatrix'),
phenoData = phenotypeDataframe,
featureData = featureDataFrame,
##lowerDetectionLimit=0.1,
expressionFamily = negbinomial.size())
writeLines(" Created Monocle object")

writeLines(" Estimate Size Factors")
cds <- estimateSizeFactors(cds)
writeLines(" Estimate Dispersions then removing outlines: ")
cds <- estimateDispersions(cds)
disp_table <- dispersionTable(cds)
writeLines(" Filter genes by mean_expression")
ordering_genes <- subset(disp_table, mean_expression >= SG_MEAN_EXPRESSION_FILTER)
cds <- setOrderingFilter(cds, ordering_genes$gene_id)
cds <- detectGenes(cds, min_expr = SG_MEAN_EXPRESSION_FILTER)
writeLines(" Reduce Dimensions: can take some time...")
cds <- reduceDimension(cds, norm_method = "log", max_components = 2, verbose = F)
writeLines(" Order cell trajectories in pseudotime.")
cds <- orderCells(cds)

writeLines(" Getting detected genes above SG_NUM_CELLS_EXPRESSION_FILTER")
expressed_genes <- row.names(subset(fData(cds), fData(cds)$num_cells_expressed >= SG_NUM_CELLS_EXPRESSION_FILTER))

writeLines(" Calculating differential expression using Pseudotime...")
diff_test_res <- differentialGeneTest(cds[expressed_genes,],
fullModelFormulaStr = "~sm.ns(Pseudotime)", cores = detectCores())

diff_genes_Pseudo <- row.names (subset(diff_test_res, qval < SG_QVAL_THRESHOLD))
diff_test_resTmp = diff_test_res[diff_genes_Pseudo,]
gene_used_order = diff_test_resTmp[diff_test_resTmp[, "use_for_ordering"] == "TRUE",]
genes_order_Pval = gene_used_order[, c(5, 4)]

writeLines(" Gather Pseudotime analysis for export")
pseudoXY <- as.data.frame(t(cds@reducedDimS))
colnames(pseudoXY) <- c(paste0("Monocle_X_", "SG_RUNID"), paste0("Monocle_Y_", "SG_RUNID"))
outState <- pData(cds)
rownames(pseudoXY) <- cellAnnotations$Sample
writeLines(" Format data output and prepare for images")
colnames(outState)[which(names(outState) == "Sample")] <- "CellId"
colnames(outState)[which(names(outState) == "Size_Factor")] <- paste0("Monocle_Size_Factor_", "SG_RUNID")
colnames(outState)[which(names(outState) == "num_genes_expressed")] <- paste0("Monocle_Genes_Expressed_", "SG_RUNID")
colnames(outState)[which(names(outState) == "Pseudotime")] <- paste0("Monocle_Pseudotime_", "SG_RUNID")
colnames(outState)[which(names(outState) == "State")] <- paste0("Monocle_State_", "SG_RUNID")
outFile = cbind(outState, pseudoXY)

writeLines(" Plot heatmap for all clusters")
outFile[, 1] <- gsub("AppID_*", "", rownames(outFile))

mynamestheme <- theme(plot.title = element_text(family = "Helvetica", face = "bold", size = (15)),
legend.title = element_text(colour = "steelblue", face = "bold.italic", family = "Helvetica"),
legend.text = element_text(face = "italic", colour = "steelblue4", family = "Helvetica"),
axis.title = element_text(family = "Helvetica", size = (10), colour = "steelblue4"),
axis.text = element_text(family = "Courier", colour = "cornflowerblue", size = (10)))


maxVal = max(as.numeric(as.character(outFile[, 5])))
if (TRUE) {
    showNames = T
    if (length(rownames(genes_order_Pval)) < 100)
    {
        showNames = T
    }
    plotHeight = length(rownames(genes_order_Pval)) * 30
    if(plotHeight > 800){
        plotHeight = 800
    }
    png(filename = paste0("SG_OUTPUT_DIRECTORY_PATH", "Pseudotime_Heatmap_SG_RUNID.png"), width = 800, height = 800)
    plotHM = plot_pseudotime_heatmap(cds[rownames(genes_order_Pval),],
    num_clusters = maxVal,
    cores = detectCores(),
    show_rownames = showNames,
    return_heatmap = T)
    dev.off()

    t <- as.data.frame(cutree(plotHM$tree_row, k = maxVal))
    colnames(t) <- "Cluster"
    t$Gene <- rownames(t)
    t = t[order(t$Cluster),]
}

write.table(t[, c(2, 1)], file = paste0("SG_OUTPUT_DIRECTORY_PATH", "GeneSets_SG_RUNID.csv"), sep = ',', quote = F, row.names = F)

write.csv(outFile, file = paste0("SG_OUTPUT_DIRECTORY_PATH", "Monocle_Analysis.csv"), quote = FALSE, row.names = FALSE)
writeLines(" Wrote pseudotime, state and other factors to CSV, writting DE Genes for pseudotime")

write.csv(maxVal, file = paste0("SG_OUTPUT_DIRECTORY_PATH", "States_SG_RUNID.csv"), row.names = F, col.names = F)
write.table(diff_genes_Pseudo, file = paste0("SG_OUTPUT_DIRECTORY_PATH", "DE_GeneSet_SG_RUNID.csv"), quote = F, row.names = F, col.names = F)

fileName = paste0("SG_OUTPUT_DIRECTORY_PATH", "DOGS_Pseudotime_SG_RUNID.csv")
write.table("[Metadata]", file = fileName, append = F, quote = F, row.names = F, col.names = F)
write.table("$DataType,DerivedObservations", file = fileName, append = T, quote = F, row.names = F, col.names = F)
write.table("$Organization,ParamsInRows", file = fileName, append = T, quote = F, row.names = F, col.names = F)
write.table("$ParamTypes,Iv", file = fileName, append = T, quote = F, row.names = F, col.names = F)
write.table("$EventTypes,Im", file = fileName, append = T, quote = F, row.names = F, col.names = F)
write.table(" ", file = fileName, append = T, quote = F, row.names = F, col.names = F)
write.table("[Data]", file = fileName, append = T, quote = F, row.names = F, col.names = F)
write.table("Gene, Monocle_QValue_SG_RUNID", file = fileName, append = T, quote = F, row.names = F, col.names = F)
write.table(genes_order_Pval, file = fileName, sep = ',', append = T, quote = F, row.names = F, col.names = F)

writeLines(" Plot pseudotime trajectories for genes used in trajectory building")


## Removed as we are having issues with SeqGeq saving images in windows
if(FALSE){

if (TRUE) {
for (i in 1 : maxVal)
{
currentGenes = t[which(t$Cluster == i),]
geneList = currentGenes$Gene
geneListLength = length(geneList)
filePath = file.path("SG_OUTPUT_DIRECTORY_PATH", paste0("Pseudotime_State_", i))
dir.create(filePath)
setwd(filePath)

if (geneListLength > 1) {
if (geneListLength > 30)
{
genes_order_PvalTMP = genes_order_Pval[geneList,]
genes_order_PvalTMP = genes_order_PvalTMP[order(genes_order_PvalTMP$qval, decreasing = F),][1 : 10,]
geneList = rownames(genes_order_PvalTMP)
}
plotHeight = length(geneList) * 200

if(plotHeight > 800){
plotHeight = 800
}

print("gene list length")
print(geneListLength)
png(filename = file.path(filePath, paste0("Pseudotime_Trajectory_SG_RUNID_", i, ".png")), width = 400, height = 800)
print(plot_genes_branched_pseudotime(cds[geneList,],
branch_point = 1,
color_by = "State",
ncol = 1) +
mynamestheme +
labs(title = paste0("Pseudotime_Trajectory_Cluster_", i), y = "Expression", x = "Pseudotime"))
dev.off()
writeLines(paste0(" Printed heatmap for cluster ", i))
}else {
writeLines(paste0(" Printed SCATTER PLOT for cluster ", paste0("Pseudotime_Trajectory_SG_RUNID_", i, ".png")))
expressionCol = t(as.matrix(exprs(cds[geneList,])))
colnames(expressionCol) = "Expression"
pseudoCol = as.data.frame(pData(cds)$Pseudotime)
colnames(pseudoCol) = "Pseudotime"
stateCol = as.data.frame(pData(cds)$State)
colnames(stateCol) = "State"
State_ID = stateCol$State
databind = as.data.frame(cbind(expressionCol, pseudoCol))
png(filename = file.path(filePath, paste0("Pseudotime_Trajectory_SG_RUNID_", i, ".png")), width = 400, height = 350)
print(
ggplot(databind, aes(x = Pseudotime, y = Expression), log="y") +
geom_smooth(method="loess", se=F, fullrange=T, fill=NA, colour="black", size=0.45) + geom_point(aes(colour=State_ID))  +  scale_y_continuous(trans='log10')
+
theme(legend.position="bottom")+
mynamestheme +
labs(y = "Expression", x = "Pseudotime")+
ggtitle(paste0("Pseudotime_Trajectory_Cluster_", i)) + theme(plot.title = element_text(hjust = 0.5))
)
dev.off()
}

plotWidth = maxVal * 100
if(plotWidth > 800 ){
plotWidth = 800
}
plotHeight = length(geneList) * 200
my_genes <- cds[row.names(subset(fData(cds), gene_short_name %in% geneList)),]
png(filename = file.path(filePath, paste0("Pseudotime_Violin_SG_RUNID_", i, ".png")), width = plotWidth, height = plotHeight)
print(plot_genes_violin(my_genes, grouping = "State", ncol = 1, plot_trend = T, color_by = "State"))
dev.off()
writeLines(paste0(" Printed Violin for cluster ", i))
}
}
png(filename = paste0("SG_OUTPUT_DIRECTORY_PATH", "Pseudotime_Branched_SG_RUNID.png"), width = 400, height = 400)
print(plot_spanning_tree(cds, backbone_color = "State", show_cell_names = F, show_branch_points = T))
dev.off()

}


writeLines(" Finished plugin Monocle, returning to SeqGeq.")